-- stream.lua
local sys = require "luci.sys"
local utl = require "luci.util"
local fs = require "nixio.fs"
local uci = require("luci.model.uci").cursor()
local parser = require "tlt_parser_lua"

local DEVICE_TYPE = 'Router'

function restart(r)
	local deviceId = r:value(2)
	c8y:send('108,'..deviceId..',SUCCESSFUL')
	sys.reboot()
end

function sysupgrade(r)
	local result
	local deviceId = r:value(2)
	local url = r:value(3)
	local username = uci:get("cot", "cumulocity", "username")
	local password = uci:get("cot", "cumulocity", "password")

	c8y:send('108,'..deviceId..',EXECUTING')

	result = sys.call("/usr/bin/curl -y 60 -u '"..username.."':'"..password.."' -o /tmp/cot_firmware.bin "..url)
	if not result or result ~= 0 then
		srInfo('Failed to download firmware file')
		c8y:send('108,'..deviceId..',FAILED')
		fs.remove("/tmp/cot_firmware.bin")
		return
	end

	result = sys.call("/sbin/sysupgrade -T /tmp/cot_firmware.bin")
	if not result or result ~= 0 then
		srInfo('Failed to verify firmware image file')
		c8y:send('108,'..deviceId..',FAILED')
		fs.remove("/tmp/cot_firmware.bin")
    else
		c8y:send('108,'..deviceId..',SUCCESSFUL')
		fork_exec("sleep 1; /etc/init.d/dropbear stop; /etc/init.d/uhttpd stop; sleep 1; /sbin/sysupgrade /tmp/cot_firmware.bin")
    end
end

function init()
	local DEVICE_NAME = uci:get("system", "system", "routername")
	local INTERVAL = uci:get("cot", "cumulocity", "interval")
	srInfo('*** Stream Init ***')

	-- set device name and type
	c8y:send('103,'..c8y.ID..','..DEVICE_NAME..','..DEVICE_TYPE)
	c8y:send('111,'..c8y.ID..','..INTERVAL)

	-- set imei, cellid and iccid first time
	mobileDataStream()

	-- create timer which will stream mobile info data
	local m_timer = c8y:addTimer(10 * 1000, 'mobileDataStream')
	m_timer:start()

	-- register restart handler
	c8y:addMsgHandler(502, 'restart')
	c8y:addMsgHandler(503, 'sysupgrade')
	return 0
end

function mobileDataStream()
	local modem = uci:get("cot", "cumulocity", "modem")
	local call = {modem = modem, signal="get_signal" }
	local results = parser:gsm(call)

	-- send mobile signal info
	c8y:send('105,'..c8y.ID..','.. string.gsub(results.signal, "%s", ""))

	local wantype = utl.ubus("file", "exec", { command="/sbin/wan_info", params={"state"} } )
	local wanip = utl.ubus("file", "exec", { command="/sbin/wan_info", params={"ip"} } )

	-- send wan info
	c8y:send('106,'..c8y.ID..','.. string.gsub(wanip.stdout, "%s", "") ..','.. string.gsub(wantype.stdout, "%s", ""))
end

function fork_exec(command)
	local pid = nixio.fork()
	if pid > 0 then
		return
	elseif pid == 0 then
		-- change to root dir
		nixio.chdir("/")

		-- patch stdin, out, err to /dev/null
		local null = nixio.open("/dev/null", "w+")
		if null then
			nixio.dup(null, nixio.stderr)
			nixio.dup(null, nixio.stdout)
			nixio.dup(null, nixio.stdin)
			if null:fileno() > 2 then
				null:close()
			end
		end

		-- replace with target command
		nixio.exec("/bin/sh", "-c", command)
	end
end
