--[[
LuCI - Lua Configuration Interface

Copyright 2008 Steven Barth <steven@midlink.org>
Copyright 2008 Jo-Philipp Wich <xm@leipzig.freifunk.net>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0
]]--

local fs = require("nixio.fs")
local ut = require "luci.util"
local md = require "luci.model.modem"

m = Map("cot")

s = m:section(NamedSection, "cumulocity", "iot", translate("Cloud of Things"))
s.addremove = false

o = s:option(Flag, "enabled", translate("Enable"),
	translate("Enable Cloud of Things Application"))
o.default= "0"
o.rmempty = false

o = s:option(Value, "server", translate("Server Address"),
	translate("Cloud of Things Server Address"))
o.rmempty = false
o.datatype = "host"

o = s:option(Value, "interval", translate("Interval"),
	translate("Push connection interval in minutes"))
o.datatype = "uinteger"
o.rmempty = false

local modem_list = md:get_all_modems()
if #modem_list > 0 then
	o = s:option(ListValue, "iface", translate("Mobile Interface"), translate("Interface name for which gsm values would be selected"))

	local modems = false

	function modem_match(modem_list, id)
		for _,v in pairs(modem_list) do
			if v.id == id then
				return true
			end
		end
		return false
	end

	m.uci:foreach("network", "interface", function(s)
		if s.modem and modem_match(modem_list, s.modem) then
			o:value(s[".name"])
			modems = true
		end
	end)

	if not modems then
		o:value("", translate("No Modem Connected"))
	end
end

function o.write(self, section, value)
	local modem = m.uci:get("network", value, "modem")
	m.uci:set(self.config, section, "modem", modem)
	m.uci:set(self.config, section, "iface", value)
end

o = s:option(Button, "reset_auth", translate("Reset Auth"),
	translate("Reset authentication data so that device could be \
			re-registered on Cloud of Things Device Management"))

function o.write()
	m.message = translate("Successfully cleared authentication data. \
			Now you can re-register device on Cloud of Things Device Management.")
	m.uci:delete("cot", "cumulocity", "tenant")
	m.uci:delete("cot", "cumulocity", "username")
	m.uci:delete("cot", "cumulocity", "password")
	m.uci:commit("cot")
	ut.ubus("file", "exec", { command="/bin/rm", params={"-f", "/tmp/coauth", "2>", "/dev/null"} } )
	ut.ubus("file", "exec", { command="/etc/init.d/cot", params={"restart"} } )
end

return m
